// MD-R
// version 1.0
// September 03, 2015
// Author: Vi Q. Ha

/// Notes: 
/// This is a template on how to create a customized output function
/// This example prints Mean Square Displacement of all particles 
/// Users will only need to change where there's a "[INSTRUCTION]:" tag
/// Everything else is meant to be left alone.

#ifndef VMD_OUT_TEMPL
#define VMD_OUT_TEMPL

#include "pointers.h"
#include "MDR.h"

class CustomOutput : protected mdr::Pointers {

    private:
		
    public:
		CustomOutput(mdr::MDR * _vmd): Pointers(_vmd) {}
		void print_msd();

};


// Edited By Vi on 2014/12/02
void CustomOutput::print_msd(){
	double MSD = 0;

	/// Select all local agents
	std::set<AGENT_TYPE*> agents;
	context->selectAgents(repast::SharedContext<AGENT_TYPE>::LOCAL, context->size(), agents);
	std::set<AGENT_TYPE*>::iterator agentToEval = agents.begin();
	
	dinner::Vector<long double> dr;

	while(agentToEval != agents.end()){
		
		/// [INSTRUTIONS]: /////////////////////////////////////////////
		/// Calculating MSD:
		dr = (*agentToEval)->true_position - (*agentToEval)->original_position;
		MSD += dr.dot(dr);

		agentToEval++;
	}
	
		
	/// [INSTRUTIONS]: /////////////////////////////////////////////////
	/// summing across processors
	double MSD_GLOBAL_SUM;
	reduce(*vmd->comm, MSD, MSD_GLOBAL_SUM, std::plus<double>() , 0);

	
	/// [INSTRUTIONS]: /////////////////////////////////////////////////
	/// printing with the main processor.
	if ( vmd->world_rank() == 0 ) 
	{
		char *buf = new char[100];

		sprintf (buf, "output/MSD/MSD.out" );

		std::ofstream out;
		double timeNow = vmd->tick() * parameters->deltaT;

		out.open(buf,std::ios::app);

		double MSD_avg = MSD_GLOBAL_SUM/(vmd->parameters->nTotal);
		out  << timeNow << " " <<  MSD_avg << std::endl;

		out.close();
	}
	
}



#endif
