// MD-R
// version 1.0
// September 03, 2015
// Author: Vi Q. Ha

/// Notes: 
/// This is a template on how to create a customized potential function
/// This example uses the Lenard Jones interatomic potential
/// Users will only need to change where there's a "[INSTRUCTION]:" tag
/// Everything else is meant to be left alone.

#ifndef VMD_POT_TEMPL
#define VMD_POT_TEMPL


#include "pairPotential.h"

class CustomPotential : public mdr::PairPotential
{
    private:
		bool isRepulsive;
		
    public:
		CustomPotential(mdr::MDR * _vmd):PairPotential(_vmd){isRepulsive = false;}
		~CustomPotential(){}
		PAIR_LIST_TYPE apply(PAIR_LIST_TYPE * NeighborList );
		void set_attractive(){isRepulsive = false;}
		void set_repulsive(){isRepulsive = true;}

};

PAIR_LIST_TYPE CustomPotential::apply(PAIR_LIST_TYPE * NeighborList)
{
	
	PAIR_LIST_TYPE FilteredList;
	FilteredList.clear();
	std::vector<repast::AgentId> Id_Pair(2);
	
	typedef dinner::Vector<long double> Vec;
	
	double local_virSum = 0.;
	double local_uSum = 0.;

	dinner::Line<double> worldSize =vmd->parameters->worldSize;

	double buffer_size = vmd->parameters->buffer_size;

	/// Self Rank Identifying 
	int rank =  vmd->rank();
	
	/// Find current tick // 2014.05.07 Vi's edit
	double t = vmd->tick();
	double neighbor_radius = vmd->parameters->rCut;

    ParticleAgent * pride , * envy;
    
	std::vector< std::vector<repast::AgentId> >::iterator it;
    it = NeighborList->begin();
    
    

    while(it != NeighborList->end())
	{
		

		pride = vmd->context->getAgent((*it)[0]);
		envy = vmd->context->getAgent((*it)[1]);
		if (pride!=0&&envy!=0){
			bool is_specified = check_specificity( (pride)->getType(),  (envy)->getType());
			if (is_specified) {
				
				/// [INSTRUTIONS]: /////////////////////////////////////
				/// Everything within this box is customizable 
				dinner::Vector<long double> r1, r2, dr;
				r1 = (pride)->position;
				r2 = (envy )->position;
				
				dr = r1 - r2;
					
				if (dr.x < -(worldSize.dx()/2)) {dr.x += worldSize.dx();}
				if (dr.x > (worldSize.dx()/2)) {dr.x -= worldSize.dx();}
				if (dr.y < -(worldSize.dy()/2)) {dr.y += worldSize.dy();}
				if (dr.y > (worldSize.dy()/2)) {dr.y -= worldSize.dy();}
				if (dr.z < -(worldSize.dz()/2)) {dr.z += worldSize.dz();}
				if (dr.z > (worldSize.dz()/2)) {dr.z -= worldSize.dz();}
				double rCut0;
				long double rr = dr.dot(dr);
				if (isRepulsive) {
					rCut0 = 1.122462;
				}else{
					rCut0 = neighbor_radius;
				}
				/// if within cut off radius
				if(rr <= (rCut0*rCut0)){
					long double rri = 1. / rr;
					long double rri3 = rri * rri * rri;
					/// compute and apply force:
					long double fcVal = 48. * rri3 * (rri3 -0.5) * rri;
					(pride)->acceleration = (pride )->acceleration + fcVal * dr;
					(envy)->acceleration = (envy)->acceleration - fcVal * dr;
					/// adjust potentual energy
					local_uSum += 4. * rri3 * (rri3 - 1.) + 1.;
					local_virSum += fcVal * rr;
				}
				/// End custom box /////////////////////////////////////
				
				
			} else {
				
				
				
					Id_Pair[0] = (*it)[0];
					Id_Pair[1] = (*it)[1];
					FilteredList.push_back (Id_Pair);
				
			}
		
		} else {
			vmd->properties->set_nebrNow(1);
		}

		it++;
	}


	local_uSum *= 0.5;
	local_virSum *= 0.5;
	vmd->properties->uSum += local_uSum;
	vmd->properties->virSum += local_virSum;


	return FilteredList;

}
#endif
